package be.iminds.ilabt.jfed.restapi.json;

import be.iminds.ilabt.jfed.util.GeniUrn;
import com.fasterxml.jackson.annotation.*;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.net.URL;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties("@context")
public class User {
    private static final Logger LOG = LoggerFactory.getLogger(User.class);

    private final String name;
    private final GeniUrn urn;
    private final String authorityTld;
    private final URL authorityServer;

    private final List<Slice> slices;
    private final List<Project> projects;

    private final URI uri;

    @JsonCreator
    User(@JsonProperty("name") String name,
         @JsonProperty("urn") GeniUrn urn,
         @JsonProperty("authorityTld") String authorityTld,
         @JsonProperty("authorityServer") URL authorityServer,
         @JsonProperty("slices") List<Slice>  slices,
         @JsonProperty("projects") List<Project> projects,
         @JsonProperty("@id") URI uri) {
        this.name = name;
        this.urn = urn;
        this.authorityTld = authorityTld;
        this.authorityServer = authorityServer;
        this.slices = slices == null ? null : Collections.unmodifiableList(new ArrayList<>(slices));
        this.projects = projects == null ? null : Collections.unmodifiableList(new ArrayList<>(projects));
        this.uri = uri;
    }

    @JsonProperty
    public String getName() {
        return name;
    }

    @JsonProperty
    @JsonSerialize(using = JsonGeniUrnSerializer.class)
    @JsonDeserialize(using = JsonGeniUrnDeserializer.class)
    public GeniUrn getUrn() {
        return urn;
    }

    @JsonProperty
    public String getAuthorityTld() {
        return authorityTld;
    }

    @JsonProperty
    public URL getAuthorityServer() {
        return authorityServer;
    }

    @JsonProperty
    public List<Slice> getSlices() {
        return slices;
    }

    @JsonProperty
    public List<Project> getProjects() {
        return projects;
    }


    @JsonProperty(value="@id")
    public URI getUri() {
        return uri;
    }

    @JsonProperty(value="@type")
    public String getClassName() {
        return getClass().getSimpleName();
    }

    @JsonProperty(value="@type")
    private void setClassName(String className) {
        if (!getClass().getSimpleName().equals(className))
            throw new IllegalArgumentException("@type must be "+getClass().getSimpleName());
    }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(this);
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+getClass().getSimpleName()+" to JSON: "+e.getMessage();
        }
    }
}
