package be.iminds.ilabt.jfed.restapi.debugdata;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.FedmonFullObjectLinkDeserializer;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.FedmonFullObjectLinkSerializer;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * ExperimentDD
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties("@context")
public class ExperimentDD implements DebugData {
    private static final Logger LOG = LoggerFactory.getLogger(ExperimentDD.class);

    private final Long id;
    private final GeniUrn sliceUrn;
    private final GeniUrn userUrn;
    private final List<ServerResourceMethodDD> serverResourceMethodDDs;

    @JsonCreator
    public ExperimentDD(@JsonProperty("id") Long id,
                        @JsonProperty("sliceUrn") GeniUrn sliceUrn,
                        @JsonProperty("userUrn") GeniUrn userUrn,
                        @JsonProperty("serverResourceMethodDDs") List<ServerResourceMethodDD> serverResourceMethodDDs) {
        this.id = id;
        this.sliceUrn = sliceUrn;
        this.userUrn = userUrn;
        this.serverResourceMethodDDs = serverResourceMethodDDs == null ? null : Collections.unmodifiableList(new ArrayList<>(serverResourceMethodDDs));
    }

    @JsonProperty
    public Long getId() {
        return id;
    }

    @JsonProperty
    @JsonDeserialize(using = GeniUrn.JsonDeserializer.class)
    @JsonSerialize(using = GeniUrn.JsonSerializer.class)
    public GeniUrn getSliceUrn() {
        return sliceUrn;
    }

    @JsonProperty
    @JsonDeserialize(using = GeniUrn.JsonDeserializer.class)
    @JsonSerialize(using = GeniUrn.JsonSerializer.class)
    public GeniUrn getUserUrn() {
        return userUrn;
    }

    @JsonProperty
    public List<ServerResourceMethodDD> getServerResourceMethodDDs() {
        return serverResourceMethodDDs;
    }

    @JsonProperty(value="@type")
    public String getClassName() {
        return getClass().getSimpleName();
    }

    @JsonProperty(value="@type")
    private void setClassName(String className) {
        if (!getClass().getSimpleName().equals(className))
            throw new IllegalArgumentException("@type must be "+getClass().getSimpleName());
    }

    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writerWithDefaultPrettyPrinter().writeValueAsString(this);
        } catch (JsonProcessingException e) {
            LOG.error("Exception converting "+getClass().getSimpleName()+" to JSON", e);
            return "Exception converting "+ getClass().getSimpleName() +" to JSON: "+e.getMessage();
        }
    }
}
